@@ -14,39 +14,48 @@ module Criterion.Internal
     (
       runAndAnalyse
     , runAndAnalyseOne
-    , runNotAnalyse
+    , runOne
+    , runFixedIters
     ) where
 
+import Control.Applicative ((<$>))
 import Control.DeepSeq (rnf)
 import Control.Exception (evaluate)
 import Control.Monad (foldM, forM_, void, when)
 import Control.Monad.Reader (ask, asks)
-import Control.Monad.Trans (liftIO)
+import Control.Monad.Trans (MonadIO, liftIO)
 import Control.Monad.Trans.Except
 import Data.Binary (encode)
 import Data.Int (Int64)
 import qualified Data.ByteString.Lazy as L
 import Criterion.Analysis (analyseSample, noteOutliers)
-import Criterion.IO (header, hGetReports)
+import Criterion.IO (header, hGetRecords)
 import Criterion.IO.Printf (note, printError, prolix, writeCsv)
 import Criterion.Measurement (runBenchmark, secs)
 import Criterion.Monad (Criterion)
 import Criterion.Report (report)
 import Criterion.Types hiding (measure)
 import qualified Data.Map as Map
+import Data.Vector (Vector)
 import Statistics.Resampling.Bootstrap (Estimate(..))
 import System.Directory (getTemporaryDirectory, removeFile)
 import System.IO (IOMode(..), SeekMode(..), hClose, hSeek, openBinaryFile,
                   openBinaryTempFile)
 import Text.Printf (printf)
 
--- | Run a single benchmark and analyse its performance.
-runAndAnalyseOne :: Int -> String -> Benchmarkable -> Criterion Report
-runAndAnalyseOne i desc bm = do
+-- | Run a single benchmark.
+runOne :: Int -> String -> Benchmarkable -> Criterion DataRecord
+runOne i desc bm = do
   Config{..} <- ask
   (meas,timeTaken) <- liftIO $ runBenchmark bm timeLimit
   when (timeTaken > timeLimit * 1.25) .
     void $ prolix "measurement took %s\n" (secs timeTaken)
+  return (Measurement i desc meas)
+
+-- | Analyse a single benchmark.
+analyseOne :: Int -> String -> Vector Measured -> Criterion DataRecord
+analyseOne i desc meas = do
+  Config{..} <- ask
   _ <- prolix "analysing with %d resamples\n" resamples
   erp <- runExceptT $ analyseSample i desc meas
   case erp of
@@ -81,7 +90,7 @@ runAndAnalyseOne i desc bm = do
              (round (ovFraction * 100) :: Int) wibble
         return ()
       _ <- note "\n"
-      return rpt
+      return (Analysed rpt)
       where bs :: (Double -> String) -> String -> Estimate -> Criterion ()
             bs f metric Estimate{..} =
               note "%-20s %-10s (%s .. %s%s)\n" metric
@@ -89,10 +98,11 @@ runAndAnalyseOne i desc bm = do
                    (if estConfidenceLevel == 0.95 then ""
                     else printf ", ci %.3f" estConfidenceLevel)
 
--- | Determine whether an Environment benchmark should be run.
-shouldRunEnv :: (String -> Bool) -> String -> (s -> Benchmark) -> Bool
-shouldRunEnv p pfx mkbench =
-  any (p . addPrefix pfx) . benchNames . mkbench $ undefined
+-- | Run a single benchmark and analyse its performance.
+runAndAnalyseOne :: Int -> String -> Benchmarkable -> Criterion DataRecord
+runAndAnalyseOne i desc bm = do
+  Measurement _ _ meas <- runOne i desc bm
+  analyseOne i desc meas
 
 -- | Run, and analyse, one or more benchmarks.
 runAndAnalyse :: (String -> Bool) -- ^ A predicate that chooses
@@ -100,7 +110,7 @@ runAndAnalyse :: (String -> Bool) -- ^ A predicate that chooses
                                   -- name.
               -> Benchmark
               -> Criterion ()
-runAndAnalyse p bs' = do
+runAndAnalyse select bs = do
   mbRawFile <- asks rawDataFile
   (rawFile, handle) <- liftIO $
     case mbRawFile of
@@ -112,28 +122,14 @@ runAndAnalyse p bs' = do
         return (file, handle)
   liftIO $ L.hPut handle header
 
-  let go !k (pfx, Environment mkenv mkbench)
-          | shouldRunEnv p pfx mkbench = do
-              e <- liftIO $ do
-                     ee <- mkenv
-                     evaluate (rnf ee)
-                     return ee
-              go k (pfx, mkbench e)
-          | otherwise = return (k :: Int)
-      go !k (pfx, Benchmark desc b)
-          | p desc'   = do _ <- note "benchmarking %s\n" desc'
-                           rpt <- runAndAnalyseOne k desc' b
-                           liftIO $ L.hPut handle (encode rpt)
-                           return $! k + 1
-          | otherwise = return (k :: Int)
-          where desc' = addPrefix pfx desc
-      go !k (pfx, BenchGroup desc bs) =
-          foldM go k [(addPrefix pfx desc, b) | b <- bs]
-  _ <- go 0 ("", bs')
+  for select bs $ \idx desc bm -> do
+    _ <- note "benchmarking %s\n" desc
+    rpt <- runAndAnalyseOne idx desc bm
+    liftIO $ L.hPut handle (encode rpt)
 
   rpts <- (either fail return =<<) . liftIO $ do
     hSeek handle AbsoluteSeek 0
-    rs <- hGetReports handle
+    rs <- fmap (map (\(Analysed r) -> r)) <$> hGetRecords handle
     hClose handle
     case mbRawFile of
       Just _ -> return rs
@@ -143,28 +139,40 @@ runAndAnalyse p bs' = do
   junit rpts
 
 -- | Run a benchmark without analysing its performance.
-runNotAnalyse :: Int64            -- ^ Number of loop iterations to run.
+runFixedIters :: Int64            -- ^ Number of loop iterations to run.
               -> (String -> Bool) -- ^ A predicate that chooses
                                   -- whether to run a benchmark by its
                                   -- name.
               -> Benchmark
               -> Criterion ()
-runNotAnalyse iters p bs' = goQuickly "" bs'
-  where goQuickly :: String -> Benchmark -> Criterion ()
-        goQuickly pfx (Environment mkenv mkbench)
-            | shouldRunEnv p pfx mkbench = do
-                e <- liftIO mkenv
-                goQuickly pfx (mkbench e)
-            | otherwise = return ()
-        goQuickly pfx (Benchmark desc b)
-            | p desc'   = do _ <- note "benchmarking %s\n" desc'
-                             runOne b
-            | otherwise = return ()
-            where desc' = addPrefix pfx desc
-        goQuickly pfx (BenchGroup desc bs) =
-            mapM_ (goQuickly (addPrefix pfx desc)) bs
+runFixedIters iters select bs =
+  for select bs $ \_idx desc bm -> do
+    _ <- note "benchmarking %s\n" desc
+    liftIO $ runRepeatedly bm iters
 
-        runOne (Benchmarkable run) = liftIO (run iters)
+-- | Iterate over benchmarks.
+for :: MonadIO m => (String -> Bool) -> Benchmark
+    -> (Int -> String -> Benchmarkable -> m ()) -> m ()
+for select bs0 handle = go (0::Int) ("", bs0) >> return ()
+  where
+    go !idx (pfx, Environment mkenv mkbench)
+      | shouldRun pfx mkbench = do
+        e <- liftIO $ do
+          ee <- mkenv
+          evaluate (rnf ee)
+          return ee
+        go idx (pfx, mkbench e)
+      | otherwise = return idx
+    go idx (pfx, Benchmark desc b)
+      | select desc' = do handle idx desc' b; return $! idx + 1
+      | otherwise    = return idx
+      where desc' = addPrefix pfx desc
+    go idx (pfx, BenchGroup desc bs) =
+      foldM go idx [(addPrefix pfx desc, b) | b <- bs]
+
+    shouldRun pfx mkbench =
+      any (select . addPrefix pfx) . benchNames . mkbench $
+      error "Criterion.env could not determine the list of your benchmarks since they force the environment (see the documentation for details)"
 
 -- | Write summary JUnit file (if applicable)
 junit :: [Report] -> Criterion ()
