@@ -42,12 +42,13 @@ module Criterion.Main
     , defaultConfig
     -- * Other useful code
     , makeMatcher
+    , runMode
     ) where
 
 import Control.Monad (unless)
 import Control.Monad.Trans (liftIO)
 import Criterion.IO.Printf (printError, writeCsv)
-import Criterion.Internal (runAndAnalyse, runNotAnalyse)
+import Criterion.Internal (runAndAnalyse, runFixedIters)
 import Criterion.Main.Options (MatchType(..), Mode(..), defaultConfig, describe,
                                versionInfo)
 import Criterion.Measurement (initializeTime)
@@ -130,14 +131,21 @@ defaultMainWith :: Config
                 -> IO ()
 defaultMainWith defCfg bs = do
   wat <- execParser (describe defCfg)
-  let bsgroup = BenchGroup "" bs
+  runMode wat bs
+
+-- | Run a set of 'Benchmark's with the given 'Mode'.
+--
+-- This can be useful if you have a 'Mode' from some other source (e.g. from a
+-- one in your benchmark driver's command-line parser).
+runMode :: Mode -> [Benchmark] -> IO ()
+runMode wat bs =
   case wat of
     List -> mapM_ putStrLn . sort . concatMap benchNames $ bs
     Version -> putStrLn versionInfo
-    OnlyRun iters matchType benches -> do
+    RunIters iters matchType benches -> do
       shouldRun <- selectBenches matchType benches bsgroup
       withConfig defaultConfig $
-        runNotAnalyse iters shouldRun bsgroup
+        runFixedIters iters shouldRun bsgroup
     Run cfg matchType benches -> do
       shouldRun <- selectBenches matchType benches bsgroup
       withConfig cfg $ do
@@ -145,6 +153,7 @@ defaultMainWith defCfg bs = do
                   "StddevUB")
         liftIO initializeTime
         runAndAnalyse shouldRun bsgroup
+  where bsgroup = BenchGroup "" bs
 
 -- | Display an error message from a command line parsing failure, and
 -- exit.
