@@ -60,6 +60,7 @@ module Criterion.Types
     , KDE(..)
     , Report(..)
     , SampleAnalysis(..)
+    , DataRecord(..)
     ) where
 
 import Control.Applicative ((<$>), (<*>))
@@ -119,7 +120,7 @@ data Config = Config {
 -- | A pure function or impure action that can be benchmarked. The
 -- 'Int64' parameter indicates the number of times to run the given
 -- function or action.
-newtype Benchmarkable = Benchmarkable (Int64 -> IO ())
+newtype Benchmarkable = Benchmarkable { runRepeatedly :: Int64 -> IO () }
 
 -- | A collection of measurements made while benchmarking.
 --
@@ -384,13 +385,13 @@ data Benchmark where
 -- benchmarks to be run.
 --
 -- > setupEnv = do
--- >   let small = replicate 1000 1
--- >   big <- readFile "/usr/dict/words"
+-- >   let small = replicate 1000 (1 :: Int)
+-- >   big <- map length . words <$> readFile "/usr/dict/words"
 -- >   return (small, big)
 -- >
 -- > main = defaultMain [
 -- >    -- notice the lazy pattern match here!
--- >    env setupEnv $ \ ~(small,big) ->
+-- >    env setupEnv $ \ ~(small,big) -> bgroup "main" [
 -- >    bgroup "small" [
 -- >      bench "length" $ whnf length small
 -- >    , bench "length . filter" $ whnf (length . filter (==1)) small
@@ -399,7 +400,7 @@ data Benchmark where
 -- >      bench "length" $ whnf length big
 -- >    , bench "length . filter" $ whnf (length . filter (==1)) big
 -- >    ]
--- >  ]
+-- >  ] ]
 --
 -- __Discussion.__ The environment created in the example above is
 -- intentionally /not/ ideal.  As Haskell's scoping rules suggest, the
@@ -636,3 +637,25 @@ instance NFData Report where
       rnf reportNumber `seq` rnf reportName `seq` rnf reportKeys `seq`
       rnf reportMeasured `seq` rnf reportAnalysis `seq` rnf reportOutliers `seq`
       rnf reportKDEs
+
+data DataRecord = Measurement Int String (V.Vector Measured)
+                | Analysed Report
+                deriving (Eq, Read, Show, Typeable, Data, Generic)
+
+instance Binary DataRecord where
+  put (Measurement i n v) = putWord8 0 >> put i >> put n >> put v
+  put (Analysed r)        = putWord8 1 >> put r
+
+  get = do
+    w <- getWord8
+    case w of
+      0 -> Measurement <$> get <*> get <*> get
+      1 -> Analysed    <$> get
+      _ -> error ("bad tag " ++ show w)
+
+instance NFData DataRecord where
+  rnf (Measurement i n v) = rnf i `seq` rnf n `seq` rnf v
+  rnf (Analysed r)        = rnf r
+
+instance FromJSON DataRecord
+instance ToJSON DataRecord
